@@ -1,35 +1,23 @@
-from typing import List
-
+from typing import Dict, Union, Type, Optional
 from lark import v_args
 from elasticsearch_dsl import Q, Text, Keyword, Integer, Field
-from optimade.filtertransformers import BaseTransformer
-from optimade.server.exceptions import BadRequest
+
+from optimade.filtertransformers import BaseTransformer, Quantity
+from optimade.server.mappers import BaseResourceMapper
 
 __all__ = ("ElasticTransformer",)
 
 
-_cmp_operators = {">": "gt", ">=": "gte", "<": "lt", "<=": "lte"}
-_rev_cmp_operators = {">": "<", ">=": "<=", "<": ">", "<=": ">=", "=": "=", "!=": "!="}
-_has_operators = {"ALL": "must", "ANY": "should"}
-_length_quantities = {
-    "elements": "nelements",
-    "elements_ratios": "nelements",
-    "dimension_types": "dimension_types",
-}
-
-
-class Quantity:
-    """Class to provide information about available quantities to the transformer.
-
-    The elasticsearch transformer will use `Quantity`s to (a) do some semantic checks,
-    (b) map quantities to the underlying elastic index.
+class ElasticsearchQuantity(Quantity):
+    """Elasticsearch-specific extension of the underlying
+    [`Quantity`][optimade.filtertransformers.base_transformer.Quantity] class.
 
     Attributes:
         name: The name of the quantity as used in the filter expressions.
-        es_field: The name of the field for this quanity in elastic search, will be
+        backend_field: The name of the field for this quantity in Elasticsearch, will be
             ``name`` by default.
-        elastic_mapping_type: A decendent of an elasticsearch_dsl Field that denotes which
-            mapping was used in the elastic search index.
+        elastic_mapping_type: A decendent of an `elasticsearch_dsl.Field` that denotes which
+            mapping type was used in the Elasticsearch index.
         length_quantity: Elasticsearch does not support length of arrays, but we can
             map fields with array to other fields with ints about the array length. The
             LENGTH operator will only be supported for quantities with this attribute.
@@ -45,62 +33,145 @@ class Quantity:
             work for quantities that share the same nested object quantity.
     """
 
+    name: str
+    backend_field: Optional[str]
+    length_quantity: Optional["ElasticsearchQuantity"]
+    elastic_mapping_type: Optional[Field]
+    has_only_quantity: Optional["ElasticsearchQuantity"]
+    nested_quantity: Optional["ElasticsearchQuantity"]
+
     def __init__(
         self,
-        name,
-        es_field: str = None,
+        name: str,
+        backend_field: str = None,
+        length_quantity: "ElasticsearchQuantity" = None,
         elastic_mapping_type: Field = None,
-        length_quantity: "Quantity" = None,
-        has_only_quantity: "Quantity" = None,
-        nested_quantity: "Quantity" = None,
+        has_only_quantity: "ElasticsearchQuantity" = None,
+        nested_quantity: "ElasticsearchQuantity" = None,
     ):
+        """Initialise the quantity from its name, aliases and mapping type.
+
+        Parameters:
+            name: The name of the quantity as used in the filter expressions.
+            backend_field: The name of the field for this quantity in Elasticsearch, will be
+                ``name`` by default.
+            elastic_mapping_type: A decendent of an `elasticsearch_dsl.Field` that denotes which
+                mapping type was used in the Elasticsearch index.
+            length_quantity: Elasticsearch does not support length of arrays, but we can
+                map fields with array to other fields with ints about the array length. The
+                LENGTH operator will only be supported for quantities with this attribute.
+            has_only_quantity: Elasticsearch does not support exclusive search on arrays, like
+                a list of chemical elements. But, we can order all elements by atomic number
+                and use a keyword field with all elements to perform this search. This only
+                works for elements (i.e. labels in ``CHEMICAL_SYMBOLS``) and quantities
+                with this attribute.
+            nested_quantity: To support optimade's 'zipped tuple' feature (e.g.
+                'elements:elements_ratios HAS "H":>0.33), we use elasticsearch nested objects
+                and nested queries. This quantity will provide the field for the nested
+                object that contains the quantity (and others). The zipped tuples will only
+                work for quantities that share the same nested object quantity.
+        """
+
+        super().__init__(name, backend_field, length_quantity)
 
-        self.name = name
-        self.es_field = es_field if es_field is not None else name
         self.elastic_mapping_type = (
             Keyword if elastic_mapping_type is None else elastic_mapping_type
         )
-        self.length_quantity = length_quantity
         self.has_only_quantity = has_only_quantity
         self.nested_quantity = nested_quantity
 
-    def __repr__(self):
-        return self.name
-
 
 class ElasticTransformer(BaseTransformer):
-    """Transformer that transforms ``v0.10.1`` grammar parse trees into queries.
+    """Transformer that transforms ``v0.10.1``/`v1.0` grammar parse
+    trees into Elasticsearch queries.
+
+    Uses elasticsearch_dsl and will produce an `elasticsearch_dsl.Q` instance.
 
-    Uses elasticsearch_dsl and will produce a `Q` instance.
     """
 
-    def __init__(self, quantities: List[Quantity]):
+    operator_map = {
+        "<": "lt",
+        "<=": "lte",
+        ">": "gt",
+        ">=": "gte",
+    }
+
+    _quantity_type: Type[ElasticsearchQuantity] = ElasticsearchQuantity
+
+    def __init__(
+        self, mapper: BaseResourceMapper = None, quantities: Dict[str, Quantity] = None
+    ):
+        if quantities is not None:
+            self.quantities = quantities
+
+        super().__init__(mapper=mapper)
+
+    def _field(self, quantity: Union[str, Quantity], nested: Quantity = None) -> str:
+        """Used to unwrap from `property` to the string backend field name.
+
+        If passed a `Quantity` (or a derived `ElasticsearchQuantity`), this method
+        returns the backend field name, modulo some handling of nested fields.
+
+        If passed a string quantity name:
+        - Check that the name does not match a relationship type,
+          raising a `NotImplementedError` if it does.
+        - If the string is prefixed by an underscore, assume this is a
+          provider-specific field from another provider and simply return it.
+          The original `property` rule would have already filtered out provider
+          fields for this backend appropriately as `Quantity` objects.
+
+        Returns:
+            The field name to use for database queries.
+
         """
-        Arguments:
-            quantities: A list of :class:`Quantity`s that describe how optimade (and other)
-                quantities are mapped to the elasticsearch index.
-        """
-        self.index_mapping = {quantity.name: quantity for quantity in quantities}
-        super().__init__()
+
+        if isinstance(quantity, str):
+            if quantity in self.mapper.RELATIONSHIP_ENTRY_TYPES:
+                raise NotImplementedError(
+                    f"Unable to filter on relationships with type {quantity!r}"
+                )
+
+            # In this case, the property rule has already filtered out fields
+            # that do not match this provider, so this indicates an "other provider"
+            # field that should be passed over
+            if quantity.startswith("_"):
+                return quantity
 
-    def _field(self, quantity, nested=None):
         if nested is not None:
-            return "%s.%s" % (nested.es_field, quantity.name)
+            return "%s.%s" % (nested.backend_field, quantity.name)
 
-        return quantity.es_field
+        return quantity.backend_field
 
-    def _query_op(self, quantity, op, value, nested=None):
-        """
-        Return a range, match, or term query for the given quantity, comparison
-        operator, and value
+    def _query_op(
+        self,
+        quantity: Union[ElasticsearchQuantity, str],
+        op: str,
+        value: Union[str, float, int],
+        nested: ElasticsearchQuantity = None,
+    ) -> Q:
+        """Return a range, match, or term query for the given quantity, comparison
+        operator, and value.
+
+        Returns:
+            An elasticsearch_dsl query.
+
+        Raises:
+            BadRequest: If the query is not well-defined or is not supported.
         """
         field = self._field(quantity, nested=nested)
-        if op in _cmp_operators:
-            return Q("range", **{field: {_cmp_operators[op]: value}})
+        if op in self.operator_map:
+            return Q("range", **{field: {self.operator_map[op]: value}})
 
-        if quantity.elastic_mapping_type == Text:
+        # If quantity is an "other provider" field then use Keyword as the default
+        # mapping type. These queries should not match on anything as the field
+        # is not present in the index.
+        elastic_mapping_type = Keyword
+        if isinstance(quantity, ElasticsearchQuantity):
+            elastic_mapping_type = quantity.elastic_mapping_type
+
+        if elastic_mapping_type == Text:
             query_type = "match"
-        elif quantity.elastic_mapping_type in [Keyword, Integer]:
+        elif elastic_mapping_type in [Keyword, Integer]:
             query_type = "term"
         else:
             raise NotImplementedError("Quantity has unsupported ES field type")
@@ -116,9 +187,8 @@ class ElasticTransformer(BaseTransformer):
             return ~Q(query_type, **{field: value}) & Q("exists", field=field)
 
     def _has_query_op(self, quantities, op, predicate_zip_list):
-        """
-        Returns a bool query that combines the operator queries ():func:`_query_op`)
-        for each predicate and zipped quantity pericates combinations.
+        """Returns a bool query that combines the operator calls `_query_op`
+        for each predicate and zipped quantity predicate combination.
         """
         if op == "HAS":
             kind = "must"  # in case of HAS we do a must over the "list" of the one given element
@@ -258,7 +328,7 @@ class ElasticTransformer(BaseTransformer):
         if not isinstance(quantity, Quantity):
             raise TypeError("Only quantities can be compared to constant values.")
 
-        return self._query_op(quantity, _rev_cmp_operators[op], value)
+        return self._query_op(quantity, self._reversed_operator_map[op], value)
 
     @v_args(inline=True)
     def value_op_rhs(self, op, value):
@@ -274,9 +344,15 @@ class ElasticTransformer(BaseTransformer):
             op = "="
 
         def query(quantity):
+
+            # This is only the case if quantity is an "other" provider's field,
+            # in which case, we should treat it as unknown and try to do a null query
+            if isinstance(quantity, str):
+                return self._query_op(quantity, op, value)
+
             if quantity.length_quantity is None:
                 raise NotImplementedError(
-                    "LENGTH is not supported for %s" % quantity.name
+                    f"LENGTH is not supported for {quantity.name!r}"
                 )
             quantity = quantity.length_quantity
             return self._query_op(quantity, op, value)
@@ -368,19 +444,6 @@ class ElasticTransformer(BaseTransformer):
 
         return lambda quantity: Q("wildcard", **{self._field(quantity): wildcard})
 
-    def property(self, args):
-        # property: IDENTIFIER ( "." IDENTIFIER )*
-        quantity_name = args[0]
-
-        if quantity_name not in self.index_mapping:
-            raise BadRequest(detail=f"{quantity_name} is not a searchable quantity")
-
-        quantity = self.index_mapping.get(quantity_name, None)
-        if quantity is None:
-            quantity = Quantity(name=quantity_name)
-
-        return quantity
-
     @v_args(inline=True)
     def string(self, string):
         # string: ESCAPED_STRING
