@@ -81,7 +81,7 @@ if CONFIG.insert_test_data and CONFIG.index_links_path.exists():
         LOGGER.debug(
             "Adding Materials-Consortia providers to links from optimade.org..."
         )
-        providers = get_providers()
+        providers = get_providers(add_mongo_id=True)
         for doc in providers:
             links_coll.collection.replace_one(
                 filter={"_id": ObjectId(doc["_id"]["$oid"])},
